constructor TSearch.Create;
begin
   inherited Create;
   DO_PVS     := True;          DISABLE_PVS_FROM := 4;
   DO_IID     := True;          DISABLE_IID_FROM := 4;
   DO_FUT1    := True;          FUT_1 := 200;
   DO_FUT2    := True;          FUT_2 := 500;
   DO_RAZOR   := True;          RAZOR_LIMIT := 800;
   NULL_IN_PV := True;          ASP   := 50;
   DO_LMR     := True;          IsFailLow := False;
   DO_STATIC  := False;
   HASH_IN_PV := True;
   DO_ASP     := False;
end; // TSearch.Create

procedure DumpPath(Temp, Depth : Integer);
          var A : Integer;
          s : string;
begin
   for A := StackBound to StackIndex do
       s := s + MoveToStr(MoveStack[A]) + ' ';

       DumpForm.TreeMemo.Text := IntToStr(Depth)
     + ' ' + s + IntToStr(Temp)
     + LINE_END + DumpForm.TreeMemo.Text;
end; // DumpPath

function IsMateScore( Val : Integer ) : Boolean;
begin
   if ( Val > MATE_VALUE - 100 )
   or ( Val < -MATE_VALUE + 100 )
      then Result := True
      else Result := False;
end; // IsMateScore

function InCheck : Boolean;
begin
 Result := IsAttackedBy( Board.Side xor 1, Board.List[ ListSide ].Sqr, True )
end; // InCheck

// Function "NullOK" represents conditions in which it's
// safe to use null move pruning.

function TSearch.NullOK : Boolean;
begin
  if  ( Board.Index[WMat] + Board.Index[BMat] + 32 > 10 )
  and ( not IsAttackedBy( BLACK, WhiteKingLoc, True ) )
  and ( not IsAttackedBy( WHITE, BlackKingLoc, True ) )
      then Result := True
      else Result := False;
end; // NullOK

function TSearch.PawnToSeventh( SqTo : Integer ) : Boolean;
begin
  if  ( Board.List[ Board.Index[ SqTo ] ].Kind = PAWN )
  and ( ( SqTo div 16 = ROW_2 ) or ( SqTo div 2 = ROW_7 ) )
      then Result := True
      else Result := False;
end; // PawnToSeventh

function IsNormalMove( Flag : Integer ) : Boolean;
begin
   if ( Flag = CAPT )
   or ( Flag = PROM )
   or ( Flag = CAPTPROM )
   or ( Flag = EPCAPT )
      then Result := False
      else Result := True;
end; // IsNormalMove

procedure TSearch.UpdateKillers( Depth, SqFr, SqTo : Integer );
begin
  if ( SqFr <> Killers[ Depth, 1 ] )
  or ( SqTo <> Killers[ Depth, 2 ] )
     then begin
       Killers[ Depth, 3 ] := Killers[ Depth, 1 ];
       Killers[ Depth, 4 ] := Killers[ Depth, 2 ];
       Killers[ Depth, 1 ] := SqFr;
       Killers[ Depth, 2 ] := SqTo;
     end;
end; // UpdateKillers

function TSearch.TimeOut : Boolean;
begin
  ETi := Time;
  if  ( SecondsBetween( ETi, STi ) >= MoveTime )
  and ( Fixed > 3 )
     then begin
       if  ( IsFailLow )
       and ( AddSec > 0 )
       then begin                        // After a fail-low it  may
         MoveTime := MoveTime + AddSec;  // pay off to check whether
         AddSec:= 0;          // there is any better move,
         Result := False;     // so  the time is  doubled
         Exit;                // instead  of interrupting
       end;                   // search.
       Result := True;
       NoMem := True;

       if SecUsed > 10 * MoveTime // bugfix
          then begin
            STi := Time;      // At  midnight,  program  may  use
            Result := False;  // double time. It's not a  perfect
            NoMem := False;   // solution, but it will do.
          end;
     end
     else Result := False;
end; // TimeOut

function TSearch.CanMove : Boolean;
         var A, LocalIndex : Integer;
         LocalList : TMoveSet;
begin
  Result := False;
  GenMoves;
  LocalList := MoveSet;
  LocalIndex := MoveNo;
  for A := 1 to LocalIndex do
  begin
    MakeMove( LocalList[ A ] );
    GenCapt;
    if not KingEnPrise
       then Result := True;
    UnMakeMove;
    if Result = True
       then Exit;
  end;
end; // CanMove

function TSearch.IsStaleMate : Boolean;
begin
  GenOppMoves;
  if ( KingEnPrise )
  or ( CanMove )
     then Result := False
     else Result := True;
end; // IsStaleMate

function TSearch.IsCheckMate : Boolean;
begin
  GenOppMoves;
  if  ( KingEnPrise )
  and ( not CanMove )
      then Result := True
      else Result := False;
end; // IsCheckMate

function IsRep( Key : Integer ) : Boolean;
         var A : Integer;
begin
  Result := False;
  for A := StackIndex - 1 downto 1 do
  begin // loop
    if RepStack[ A ] = Key
       then begin
         Result := True;
         Exit;
       end;
  end; // loop
end; // IsRep

function TSearch.MinimalWindowSearch( Depth, Alfa, Beta, Target : Integer; WasNull, WasLate : Boolean ) : Integer;
         var A, Temp, Best : Integer;
         TempEval : Integer;
         LocalMoveList : TMoveSet;
         Current, BestMove : TMove;
         LocalIndex : Integer;
         HTemp, HNo : Integer;
         InitialAlfa, NewDepth : Integer;
         bFutility  : Boolean;
         bRazoring  : Boolean;
         bBlockLate : Boolean;
         label TrueSearch;
         label prune;

procedure ChangeOrder;
          var A : Integer;
          HFr, HTo : Byte;
begin
   if MainHash[ HNo ].Key = HTemp
   then begin
     HFr := MainHash[ HNo ].SqFrom;
     HTo := MainHash[ HNo ].SqTo;
   end
   else begin
     HFr := OUTSIDE;
     HTo := OUTSIDE;
   end;

   for A := 1 to LocalIndex do
   with LocalMoveList[ A ] do begin  // loop

     if  ( SqFr = Killers[ Depth, 3 ] )
     and ( SqTo = Killers[ Depth, 4 ] )
         then Order := CAPT_SORT + 100;

     if  ( SqFr = Killers[ Depth, 1 ] )
     and ( SqTo = Killers[ Depth, 2 ] )
         then Order := CAPT_SORT + 101;

     if  ( SqFr = HFr )
     and ( SqTo = HTo )
         then Order := CAPT_SORT + CAPT_SORT;
   end; // loop
end; // ChangeOrder

procedure PickMove;
          var A, Max, Where : Integer;
begin
   Max   := Low( Integer );
   Where := 1;
   for A := 1 to LocalIndex do
   if  ( LocalMoveList[ A ].Order > Max )
   and ( LocalMoveList[ A ].Flag < NOFLAG )
   then begin
     Where := A;
     Max := LocalMoveList[ A ].Order;
   end;
   Current := LocalMoveList[ Where ];
   LocalMoveList[ Where ].Flag := NOFLAG;
end; // PickMove

begin  // Search
  if Depth = Target
     then begin
      if IsAttackedBy( Board.Side, Board.List[ -ListSide ].Sqr, True )
         then begin
           Dec( Target );     // No  QS  if  in  check,  /
           goto TrueSearch;   // normal search instead.  /
         end;
      Result := Quiesc( Alfa, Beta, True );
      ThinkLine[ Depth ] := '';
     end
     else begin
       TrueSearch:
       Inc( MyStat.Nodes );

       // 1. REPETITION DETECTION
       HTemp := InitHashValue;
       if IsRep( HTemp )
          then begin
            Result := 0;
            ThinkLine[Depth] := '[rep] ';
            Exit;
          end;
       RepStack[ StackIndex ] := HTemp;

       // 2. HASH RETRIEVAL
       HNo := GetHashAddr( HTemp );
       if  ( MainHash[ HNo ].Key = HTemp )
       and ( MainHash[ HNo ].Depth >= Depth )
       then begin
         {$IFDEF FULL_STATS} Inc( MyStat.HHits ); {$ENDIF}
         if ( MainHash[ HNo ].Flag = EXACT )
         then begin
           {$IFDEF FULL_STATS} Inc( MyStat.HCuts ); {$ENDIF}
           Result := MainHash[ HNo ].Val;
           if Result > Beta
              then Result := Beta;
           Exit;
         end;

         if MainHash[ HNo ].Flag = ABOVE_BETA
            then Alfa := Max( Alfa, MainHash[ HNo ].Val );

         if MainHash[ HNo ].Flag = BELOW_ALFA
            then Beta := Min( Beta, MainHash[ HNo ].Val );

         if Alfa >= Beta
            then begin
              {$IFDEF FULL_STATS} Inc( MyStat.HCuts ); {$ENDIF}
              Result := MainHash[ HNo ].Val;
              Exit;
            end;
       end; // retrieving hash value

       if IsDraw
          then begin
            Result := 0;
            Exit;
          end;

       // 3. NULL MOVE
       if  ( Depth > Target + 2 )
       and ( not WasNull )
       and ( NullOK )
       then begin
          {$IFDEF FULL_STATS} Inc( MyStat.NullTries ); {$ENDIF}
          MakeNullMove;
          if Depth > 6
             then Temp := -MinimalWindowSearch( Depth-3, -Beta-1, -Beta+1, Target, True, True )
             else Temp := -MinimalWindowSearch( Depth-2, -Beta-1, -Beta+1, Target, True, True );
          UnMakeMove;
          if Temp >= Beta
             then begin
                {$IFDEF FULL_STATS} Inc( MyStat.NullCuts ); {$ENDIF}
                Result := Beta;
                Exit;
             end;
       end; // null move

       // 4. SETTING CONDITIONS FOR FUTILITY PRUNING
       bFutility := False;
       bRazoring := False;

       if  ( Depth = Target + 1 )
       and ( DO_FUT1 )
       and ( not IsMateScore( Alfa ) )
       and ( Fixed > 6 )
       then begin
         TempEval := MyEval.Eval( True );
         if TempEval + FUT_1 <= Alfa
            then bFutility := True;
       end; // Futility for Depth-1

       if  ( Depth = Target + 2 )
       and ( DO_FUT2 )
       and ( not IsMateScore( Alfa ) )
       and ( Fixed > 7 )
       then begin
         TempEval := MyEval.Eval( True );
         if TempEval + FUT_2 <= Alfa
            then bFutility := True;
       end; // Futility for Depth-2

       if  ( Depth = Target + 3 )
       and ( DO_RAZOR )
       and ( Fixed > 7 )
       and ( not IsMateScore( Alfa ) )
       then begin
         TempEval := MyEval.Eval( True );
         if TempEval + RAZOR_LIMIT <= Alfa
            then bRazoring := True;
       end; // Razoring

       // 5. SEARCH PREPARATION
       Best := -INFINITY;
       InitialAlfa := Alfa;

       GenMoves;
       if KingEnPrise
          then begin
            Result := MATE_VALUE + Depth;
            Exit;
          end;

       LocalMoveList := MoveSet;
       LocalIndex := MoveNo;
       ChangeOrder;

       // 6. SEARCH LOOP:
       for A := 1 to LocalIndex do
       begin // loop

         if  ( Depth > Target + 4 )
         and ( Fixed > 1 )
         then begin  // the clock
         if TimeOut  // the time is up
            then begin
              Result := -INFINITY;
              Exit;
            end
            else Application.ProcessMessages;
         end; // The clock

         bBlockLate := WasLate;
         PickMove;
         MakeMove( Current );

         // Extension and reduction heuristics

         // A. Pawn to 2nd/7th rank extended by 2 plies
         if PawnToSeventh( Current.SqTo )
         then NewDepth := Target - 2
         else begin
           // B. Check extension by 1 ply
           if  ( Target > -MaxExt )
           and ( InCheck )
           then begin  // check extension
             NewDepth := Target - 1;
             {$IFDEF FULL_STATS} Inc( MyStat.Checks ); {$ENDIF}
           end         // check extension
           else begin  // no extension, trying to reduce
             NewDepth := Target;

             // C. Futility pruning
             if  ( bFutility )
             and ( A > 3 )
             and ( IsNormalMove( Current.Flag ) )
             then begin
                Temp := Alfa;
                {$IFDEF FULL_STATS} Inc( MyStat.FutCuts ); {$ENDIF}
                goto prune;
             end; // Futility pruning

             // D. Late move reduction
             if  ( DO_LMR )
             and ( not bBlockLate )
             and ( Fixed > 5 )
             and ( Depth > Target + 1 )
             and ( A > 18 )
             then begin
               NewDepth := NewDepth + 1;
               {$IFDEF FULL_STATS} Inc( MyStat.LMReds ); {$ENDIF}
               bBlockLate := True;
             end; // Late move reduction

             // E. Razoring
             if  ( bRazoring )
             and ( A > 3 )
             and ( IsNormalMove( Current.Flag ) )
             and ( not BBlockLate )
             then begin
               {$IFDEF FULL_STATS} Inc( MyStat.RazorReds ); {$ENDIF}
               NewDepth := NewDepth + 1;
             end; // Razoring

           end; // of reductions
         end;   // of extension/reduction block

         Temp := -MinimalWindowSearch( Depth-1, -Beta, -Alfa, NewDepth, False, bBlockLate );
         prune:
         UnmakeMove;

         if  ( Temp < -9000 ) // stalemate detection
         and ( IsStaleMate )
             then Temp := 0;

         if Temp > Best
            then begin
              Best := Temp;
              BestMove := Current;
           //   ThinkLine[ Depth ] := MoveToStr( Current )
           //                       + ' ' + ThinkLine[Depth - 1];
            end;

         if Best >= Beta
            then begin
              History.Update( Depth, Current.SqFr, Current.SqTo );

              if  ( IsNormalMove( Current.Flag ) )
              or  ( ( Current.Flag = CAPT ) and ( Current.Order < CAPT_SORT + 100 ) )
              then UpdateKillers( Depth, Current.SqFr, Current.SqTo );

              Break;
            end;

         if Best > Alfa
            then Alfa := Best;

       end; // loop

       ThinkLine[ Depth ] := '';
       Result := Best;
       WriteHash( HNo, HTemp, BestMove.SqFr, BestMove.SqTo, Depth, Best, InitialAlfa, Beta );
     end;
end; // MinimalWindowSearch

// Function "Search" does principal variation search (PVS)

function TSearch.Search( Depth, Alfa, Beta, Target : Integer; WasNull : Boolean ) : Integer;
         var A, Temp, Best : Integer;
         LocalMoveList : TMoveSet;
         Current, BestMove : TMove;
         LocalIndex : Integer;
         HTemp, HNo : Integer;
         InitialAlfa, NewDepth : Integer;
         label TrueSearch;

procedure ChangeOrder;
          var A : Integer;
          HFrom, HTo : Byte;
begin
   if MainHash[ HNo ].Key = HTemp
   then begin
     HFrom := MainHash[ HNo ].SqFrom;
     HTo   := MainHash[ HNo ].SqTo;
   end
   else begin
     HFrom := OUTSIDE;
     HTo   := OUTSIDE;
   end;

   for A := 1 to LocalIndex do
   with LocalMoveList[ A ] do begin  // loop

     if  ( SqFr = Killers[ Depth, 3 ] )
     and ( SqTo = Killers[ Depth, 4 ] )
         then Order := CAPT_SORT + 100;

     if  ( SqFr = Killers[ Depth, 1 ] )
     and ( SqTo = Killers[ Depth, 2 ] )
         then Order := CAPT_SORT + 101;

     if  ( SqFr = HFrom )
     and ( SqTo = HTo )
         then Order := CAPT_SORT + CAPT_SORT;
   end; // loop
end; // ChangeOrder

procedure PickMove;
          var A, Max, Where : Integer;
begin
   Max   := Low( Integer );
   Where := 1;
   for A := 1 to LocalIndex do
   if  ( LocalMoveList[ A ].Order > Max )
   and ( LocalMoveList[ A ].Flag < NOFLAG )
   then begin
     Where := A;
     Max := LocalMoveList[ A ].Order;
   end;
   Current := LocalMoveList[ Where ];
   LocalMoveList[ Where ].Flag := NOFLAG;
end; // PickMove

begin  // Search
  if Depth = Target
     then begin
      if IsAttackedBy( Board.Side, Board.List[-ListSide].Sqr, True )
         then begin
            Dec( Target );     // No  QS  if  in  check,  /
            goto TrueSearch;   // normal search instead.  /
         end;
      Result := Quiesc( Alfa, Beta, True );
      ThinkLine[ Depth ] := '';
     end
     else begin
       TrueSearch:
       Inc( MyStat.Nodes );
       Inc( MyStat.PVNodes );

       // 1. REPETITION DETECTION:
       HTemp := InitHashValue;
       if IsRep( HTemp )
          then begin
            Result := 0;
            ThinkLine[ Depth ] := '[rep] ';
            Exit;
          end;
       RepStack[ StackIndex ] := HTemp;

       // 2. HASH RETRIEVAL:
       HNo := GetHashAddr( HTemp );
       if  ( HASH_IN_PV )
       and ( MainHash[ HNo ].Key = HTemp )
       then begin // found entry
         if ( MainHash[ HNo ].Depth >= Depth )
         then begin // correct depth
           {$IFDEF FULL_STATS} Inc( MyStat.HHits ); {$ENDIF}
           if ( MainHash[ HNo ].Flag = EXACT )
           then begin
             {$IFDEF FULL_STATS} Inc( MyStat.HCuts ); {$ENDIF}
             Result := MainHash[ HNo ].Val;
             if Result > Beta
                then Result := Beta;
             ThinkLine[Depth] := '[hash] ';
             Exit;
           end;

           if MainHash[ HNo ].Flag = ABOVE_BETA
              then Alfa := Max( Alfa, MainHash[ HNo ].Val );

           if MainHash[ HNo ].Flag = BELOW_ALFA
              then Beta := Min( Beta, MainHash[ HNo ].Val );

           if Alfa >= Beta
              then begin
                {$IFDEF FULL_STATS} Inc( MyStat.HCuts ); {$ENDIF}
                Result := MainHash[ HNo ].Val;
                ThinkLine[Depth] := '[hash]';
                Exit;
              end;
         end; // correct depth
       end // found entry
       else begin

       // No hash move found - do internal iterative    /
       // deepening. This means starting a shallower    /
       // search  in  order to create a  hash  entry    /
       // that will help in move ordering.              /

       if  ( DO_PVS )
       and ( DO_IID )
       and ( Depth > DISABLE_IID_FROM )
           then begin
             {$IFDEF FULL_STATS} Inc( MyStat.IIDeep ); {$ENDIF}
             Search( Depth-2, Alfa, Beta, 0, False );
           end;
       end;

       if IsDraw
          then begin
            Result := 0;
            Exit;
          end;

       // 3. NULL MOVE:
       if  ( NULL_IN_PV )
       and ( Depth > Target + 2 )
       and ( not WasNull )
       and ( NullOK )
       then begin
          {$IFDEF FULL_STATS} Inc( MyStat.NullTries ); {$ENDIF}
          MakeNullMove;
          if Depth > 6
          then Temp := -MinimalWindowSearch( Depth-3, -Beta-1, -Beta+1, Target, True, True )
          else Temp := -MinimalWindowSearch( Depth-2, -Beta-1, -Beta+1, Target, True, True );
          UnMakeMove;

          if Temp >= Beta
             then begin
               {$IFDEF FULL_STATS} Inc( MyStat.NullCuts ); {$ENDIF}
               Result := Beta;
               Exit;
             end;
       end; // null move

       // 4. SEARCH PREPARATION
       Best := -INFINITY;
       InitialAlfa := Alfa;

       GenMoves;
       if KingEnPrise
          then begin
            Result := MATE_VALUE + Depth;
            Exit;
          end;

       LocalMoveList := MoveSet;
       LocalIndex    := MoveNo;
       ChangeOrder;

       // 5. SEARCH LOOP:
       for A := 1 to LocalIndex do
       begin // loop

         if  ( Depth > Target + 4 )
         and ( Fixed > 1 )
         then begin // the clock
         if TimeOut  // the time is up
            then begin
               Result := -INFINITY;
               Exit;
            end
            else Application.ProcessMessages;
         end; // The clock

         PickMove;
         MakeMove( Current );

         // Extension and reductions ordered by depth:

         // A. Pawn to 2nd/7th rank extended by 2 plies
         if  PawnToSeventh( Current.SqTo )
         then NewDepth := Target - 2
         else begin
           // B. Check extension by 1 ply
           if  ( Target > -MaxExt )
           and ( InCheck )
           then begin  // check extension
             NewDepth := Target - 1;
             {$IFDEF FULL_STATS} Inc( MyStat.Checks ); {$ENDIF}
           end         // check extension
           else begin
             NewDepth := Target;
           end; // end of (possible) reductions
         end;  // end of extension/reduction block

         // Program  is using  Principle  Variation  Search,
         // except  for  nodes near the  leafs.  This  means
         // doing  a  null-window  search  around  the  best
         // score found so far, and entering a normal window
         // search only if that fails high. Of course it is
         // not applicable while searching the first move.

         if ( not DO_PVS )
         or ( A = 1 )
         or ( Depth > DISABLE_PVS_FROM )
         or ( -MinimalWindowSearch( Depth-1, -Best-1, -Best, NewDepth, False, False ) > Best )
            then Temp := -Search( Depth-1, -Beta, -Alfa, NewDepth, False );

         if  ( Temp < -9000 ) // stalemate detection
         and ( IsStaleMate )
             then Temp := 0;

         if Temp > Best
            then begin
              if  ( Form1.IsDumping )
              and ( Fixed = 7 )
              and ( Depth = 1 )
                  then DumpPath( Temp, Depth );
              Best := Temp;
              BestMove := Current;
              ThinkLine[ Depth ] := MoveToStr( Current )
                                  + ' ' + ThinkLine[Depth - 1];
            end;

         UnMakeMove;

         if Best >= Beta
            then begin
              History.Update( Depth, Current.SqFr, Current.SqTo );

              if  ( IsNormalMove(Current.Flag ) )
              or  ( ( Current.Flag = CAPT ) and ( Current.Order < CAPT_SORT + 100 ) )
              then UpdateKillers( Depth, Current.SqFr, Current.SqTo );

              Break;
           end;

        if Best > Alfa
           then Alfa := Best;
       end; // loop

       Result := Best;
       WriteHash( HNo, HTemp, BestMove.SqFr, BestMove.SqTo, Depth, Best, InitialAlfa, Beta );
     end;
end; // Search
